﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.VehicleRouting.Encodings.General;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Common;
using HeuristicLab.Problems.VehicleRouting.Encodings.Potvin;

namespace HeuristicLab.Problems.VehicleRouting.Encodings.ExtendedPotvin {
  [Item("ExtendedPotvinManipulator", "A VRP manipulation operation.")]
  [StorableClass]
  public class ExtendedPotvinManipulator : VRPManipulator, IStochasticOperator, IExtendedPotvinOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    public IValueParameter<PotvinManipulator> ToursManipuator {
      get { return (IValueParameter<PotvinManipulator>)Parameters["ToursManipuator"]; }
    }

    public IValueParameter<IPermutationManipulator> VehicleAssignmentManipuator {
      get { return (IValueParameter<IPermutationManipulator>)Parameters["VehicleAssignmentManipuator"]; }
    }

    public IValueParameter<PercentValue> VehicleAssignmentMutationRate {
      get { return (IValueParameter<PercentValue>)Parameters["VehicleAssignmentMutationRate"];}
    }

    public ILookupParameter<Permutation> VehicleAssignmentParameter {
      get { return (ILookupParameter<Permutation>)Parameters["VehicleAssignment"]; }
    }

    [StorableConstructor]
    protected ExtendedPotvinManipulator(bool deserializing) : base(deserializing) { }

    public ExtendedPotvinManipulator() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic manipulation operators."));
      Parameters.Add(new ValueParameter<PotvinManipulator>("ToursManipuator",
         "The operator used to menipulate the tours.", new PotvinCustomerRelocationMainpulator()));
      Parameters.Add(new ValueParameter<IPermutationManipulator>("VehicleAssignmentManipuator",
        "The operator used to menipulate the vehicle assignments.", new Swap2Manipulator()));
      Parameters.Add(new LookupParameter<Permutation>("VehicleAssignment"));
      Parameters.Add(new ValueParameter<PercentValue>("VehicleAssignmentMutationRate", "The mutation rate of the vehicle assignments", new PercentValue(0.1)));
    }

    protected ExtendedPotvinManipulator(ExtendedPotvinManipulator original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExtendedPotvinManipulator(this, cloner);
    }

    public override IOperation Apply()
    {
      IVRPEncoding solution = VRPToursParameter.ActualValue;
      if (!(solution is ExtendedPotvinEncoding)) {
        VRPToursParameter.ActualValue = ExtendedPotvinEncoding.ConvertFrom(solution, ProblemInstance);
      }

      OperationCollection next = new OperationCollection(base.Apply());

      ToursManipuator.Value.VRPToursParameter.ActualName = VRPToursParameter.ActualName;
      next.Insert(0, ExecutionContext.CreateOperation(ToursManipuator.Value));

      if (RandomParameter.ActualValue.NextDouble() < VehicleAssignmentMutationRate.Value.Value) {
        VehicleAssignmentParameter.ActualValue = (VRPToursParameter.ActualValue as ExtendedPotvinEncoding).VehicleAssignment;
        VehicleAssignmentManipuator.Value.PermutationParameter.ActualName = VehicleAssignmentParameter.ActualName;
        next.Insert(1, ExecutionContext.CreateOperation(VehicleAssignmentManipuator.Value));
      }

      return next;
    }
  }
}
